openerp.print_engine = function (instance, local) {
    local.SocketManager = instance.web.Class.extend({
        init: function () {
            this.socket = null;
            this.start();
        },
        start: function () {
            this.open_socket();
        },
        open_socket: function () {
            var self = this;

            var setup_socket = function (config) {
                var protocol = window.location.protocol;
                self.socket = io(protocol + '//' + config.host + ':' + config.port, {
                    path: config.path
                });
                self.socket.on('connect', self.handle_connect);
                self.socket.on('connect_error', self.handle_connect_error);
                self.socket.on('show-printers', self.handle_printers);
                self.socket.on('show-print-status', self.handle_print_status);
            }

            this.get_socket_config().then(setup_socket);
        },
        handle_connect: function () {
            $('#printer-status').removeClass();
            $('#printer-status').addClass('printer-status-online');
        },
        handle_connect_error: function () {
            $('#printer-status').removeClass();
            $('#printer-status').addClass('printer-status-offline');
        },
        handle_printers: function (printers) {
            instance.web.unblockUI();
        
            var widget = new local.PrinterSelectionWidget(printers);
            widget.get_selection().then(function (printer) {
                console.log(printer);
            });
        },
        handle_print_status: function (data) {
            instance.web.unblockUI();

            if (_.isEqual(data.status, 'printing')) {
                instance.web.notification.do_notify('Impresión', 'La impresora ' + data.printer + ' está imprimiendo');
                return;
            }

            if (_.isEqual(data.status, 'printed')) {
                instance.web.notification.do_notify('Impresión', 'La impresora ' + data.printer + ' finalizó la impresión');
                return;
            }

            if (_.isEqual(data.status, 'error')) {
                instance.web.notification.do_notify('Impresión', 'Ocurrió un error al imprimir: ' + data.printer);
            }
        },
        get_socket_config: function () {
            var url = '/print_engine/socket_config';
            return this.get_from_server(url);
        },
        get_from_server: function (url) {
            instance.web.blockUI();
            return $.Deferred(function (d) {
                instance.web.jsonRpc(url, 'call').done(function () {
                    instance.web.unblockUI();
                    d.resolve.apply(d, arguments);
                }).fail(function (e) {
                    instance.web.unblockUI();
                    d.reject.apply(d, arguments);
                });
            });
        },
        request_print: function (data) {
            if (!this.socket || !this.socket.connected) {
                instance.web.notification.do_warn('Impresión', 'La impresora no está preparada');
                return;
            }

            instance.web.blockUI();

            try {
                this.socket.emit('request-print', {
                    data: data
                });
            } catch (e) {
                instance.web.unblockUI();
                instance.webclient.crashmanager.show_message(e);
            }
        }
    });

    local.PrinterTopWidget = instance.web.Widget.extend({
        template: 'PrinterTopWidget',
        init: function (parent) {
            this._super(parent);
        },
        start: function () {
            // this.$el.click(this, this.selectDefaultPrinter);
        }
    });

    local.PrinterSelectionWidget = instance.web.Widget.extend({
        template: 'PrinterSelectionWidget',
        events: {
            'click li': 'on_select'
        },
        printers: [],
        init: function (printers) {
            this.printers = printers;
            this.start();
        },
        start: function () {
            this.defer = $.Deferred();
            this.render_widget();
        },
        render_widget: function () {
            this.renderElement();
            $('body').append(this.$el);
            
            this.$el.on('hidden.bs.modal', this, this.on_hide);

            this.$el.modal('show');
        },
        get_selection: function () {
            return this.defer;
        },
        on_select: function (e) {
            var $el = $(e.target).closest('li');
            var name = $el.data('name');
            this.selected_printer = name;
            this.$el.modal('hide');
        },
        on_hide: function (e) {
            var self = e.data;
            self.defer.resolve(self.selected_printer);
        }
    });

    if (instance.web) {
        if (instance.web.UserMenu) {
            instance.web.UserMenu.include({
                do_update: function () {
                    var printer = new local.PrinterTopWidget(this);
                    printer.appendTo($('.oe_systray'));
                    local.socket = new local.SocketManager();
                    return this._super.apply(this, arguments);
                }
            });
        }

        if (instance.web.ActionManager) {
            instance.web.ActionManager.include({
                ir_actions_report_xml: function (action, options) {
                    var url = '/print_engine/get_pdf';
                    var ctx = _.clone(action.context);

                    ctx.report_name = action.report_name;

                    instance.web.jsonRpc(url, 'call', {
                        context: ctx
                    }).done(function (result) {
                        instance.print_engine.socket.request_print(result.data);
                    }).fail(function (e) {
                        instance.webclient.crashmanager.show_message(e);
                    });
                }
            });
        }
    }
}
